use cgmath::{Vector3, Zero};

use super::ray::Ray;

pub struct AABB {
    pub bounds: [Vector3<f32>; 2],
    pub start_index: usize,
    pub end_index: usize,
}

impl AABB {
    pub fn new() -> AABB {
        AABB {
            bounds: [Vector3::zero(), Vector3::zero()],
            start_index: 0,
            end_index: 0,
        }
    }

    pub fn intersect(&self, ray: &Ray) -> bool {
        let mut tmin = (self.bounds[ray.signs.x].x - ray.origin.x) * ray.inv_direction.x;
        let mut tmax = (self.bounds[1 - ray.signs.x].x - ray.origin.x) * ray.inv_direction.x;
        let tymin = (self.bounds[ray.signs.y].y - ray.origin.y) * ray.inv_direction.y;
        let tymax = (self.bounds[1 - ray.signs.y].y - ray.origin.y) * ray.inv_direction.y;

        if tmin > tymax || tymin > tmax {
            return false;
        }

        if tymin > tmin {
            tmin = tymin;
        }

        if tymax < tmax {
            tmax = tymax;
        }

        let tzmin = (self.bounds[ray.signs.z].z - ray.origin.z) * ray.inv_direction.z;
        let tzmax = (self.bounds[1 - ray.signs.z].z - ray.origin.z) * ray.inv_direction.z;

        if tmin > tzmax || tzmin > tmax {
            return false;
        }

        if tzmin > tmin {
            tmin = tzmin;
        }

        if tzmax < tmax {
            tmax = tzmax;
        }

        let mut t = tmin;

        if t < 0.0 {
            t = tmax;

            if t < 0.0 {
                return false;
            }
        }

        true
    }
}
