use cgmath::{InnerSpace, Vector3};

use super::ray::Ray;

pub struct Camera {
    position: Vector3<f32>,
    direction: Vector3<f32>,
    up: Vector3<f32>,
    right: Vector3<f32>,

    _fov: f32,
    tan: f32,
    aspect_ratio: f32,
}

impl Camera {
    pub fn new(
        eye: Vector3<f32>,
        center: Vector3<f32>,
        up: Vector3<f32>,
        fov: f32,
        aspect_ratio: f32,
    ) -> Camera {
        let mut up = up.normalize();

        let view = (center - eye).normalize();

        let right = view.cross(up).normalize();

        up = right.cross(view);

        let fovy = fov.to_radians() / 2.0;

        Camera {
            position: eye,
            direction: view,
            up,
            right,

            _fov: fovy,
            tan: fovy.tan(),
            aspect_ratio,
        }
    }

    pub fn primary_ray(&self, x: u32, y: u32, dim_x: u32, dim_y: u32) -> Ray {
        let x_scaled = Self::scale(x, dim_x);
        let y_scaled = Self::scale(y, dim_y);

        let mut dir = self.direction;

        dir += self.right * self.tan * self.aspect_ratio * x_scaled;
        dir += self.up * self.tan * y_scaled;

        dir = dir.normalize();

        Ray::new(self.position, dir)
    }

    fn scale(v: u32, s: u32) -> f32 {
        ((v as f32 + 0.5) / s as f32) * 2.0 - 1.0
    }
}
