use cgmath::{InnerSpace, Vector3};

use std::f32::consts::PI;

pub struct View {
    pub position: Vector3<f32>,
    pub look_at: Vector3<f32>,
    pub up: Vector3<f32>,
    pub fov: f32,
}

impl View {
    /// returns `up` and `right`
    pub fn axises(&self) -> (Vector3<f32>, Vector3<f32>) {
        let view_dir = self.look_at - self.position;

        let horiz_axis = view_dir.cross(self.up).normalize();
        let vert_axis = horiz_axis.cross(view_dir).normalize();
        let right = horiz_axis * Self::scale(self.fov);
        let up = vert_axis * Self::scale(-self.fov);

        (up, right)
    }

    fn scale(fov: f32) -> f32 {
        (0.5 * fov * PI / 180.0).tan()
    }
}
